
import unittest
import lotus_core_hpp
import math
import os

def td_fock1e_K_mat_base(filename, exact_v, mode):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  if mode==0:
    fock1e = lotus_core_hpp.Fock1e()
    S      = lotus_core_hpp.dMatrix()
    K      = lotus_core_hpp.dMatrix()
  elif mode==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    S      = lotus_core_hpp.dMatrix_map()
    K      = lotus_core_hpp.dMatrix_map()

  fock1e.cal_S(S, scgtos)
  fock1e.cal_K(K, scgtos)
  sk     = lotus_core_hpp.Util.cal_DV(S, K)

  abs_error=1.0e-8
  if math.fabs(sk - exact_v)<abs_error: ret=0
  else:          
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0:
      print "exact_v ",exact_v," sk ",sk," diff ",exact_v-sk      
    ret=1
 
  return ret


def td_fock1e_NA_mat_base(filename, exact_v, mode):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  if mode==0:
    fock1e = lotus_core_hpp.Fock1e()
    S      = lotus_core_hpp.dMatrix()
    NA     = lotus_core_hpp.dMatrix()
  elif mode==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    S      = lotus_core_hpp.dMatrix_map()
    NA     = lotus_core_hpp.dMatrix_map()

  charges    = lotus_core_hpp.get_vector_Charge(2)
  charges[0] = lotus_core_hpp.Charge(1, 0.1, 0.2, 0.3, 0)
  charges[1] = lotus_core_hpp.Charge(2, 1.0, 1.1, 1.2, 1)
  fock1e.cal_S(S, scgtos)
  fock1e.cal_NA(NA, scgtos, charges)

  sna     = lotus_core_hpp.Util.cal_DV(S, NA)

  abs_error=1.0e-8
  if math.fabs(sna - exact_v)<abs_error: ret=0
  else:          
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0:
      print "exact_v ",exact_v," sna ",sna," diff ",exact_v-sna     
    ret=1
 
  return ret



def td_fock1e_ECP_mat_base(filename, exact_v, mode):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)
  if mode==0:
    fock1e = lotus_core_hpp.Fock1e()
    cecp   = lotus_core_hpp.ECP_matrix()
    S      = lotus_core_hpp.dMatrix()
    M_ecp  = lotus_core_hpp.dMatrix()
  elif mode==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    cecp   = lotus_core_hpp.ECP_matrix_map()
    S      = lotus_core_hpp.dMatrix_map()
    M_ecp  = lotus_core_hpp.dMatrix_map()

  fock1e.cal_S(S, scgtos)
  cecp.cal_ECP(M_ecp, scgtos, ecps)
  secp     = lotus_core_hpp.Util.cal_DV(S, M_ecp)

  abs_error=1.0e-8
  if math.fabs(secp - exact_v)<abs_error: ret=0
  else:          
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0:
      print "exact_v ",exact_v," secp ",secp," diff ",exact_v-secp      
    ret=1
 
  return ret


def td_fock1e_grad_base(filename, exact_v, mode, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)
  if mode_mat==0:
    fock1e = lotus_core_hpp.Fock1e()
    cecp   = lotus_core_hpp.ECP_matrix()
    S      = lotus_core_hpp.dMatrix()
  elif mode_mat==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    cecp   = lotus_core_hpp.ECP_matrix_map()
    S      = lotus_core_hpp.dMatrix_map()

  charges    = lotus_core_hpp.get_vector_Charge(2)
  charges[0] = lotus_core_hpp.Charge(1, 0.1, 0.2, 0.3, 0)
  charges[1] = lotus_core_hpp.Charge(2, 1.0, 1.1, 1.2, 1)
  fock1e.cal_S(S, scgtos)

  if mode==0:  grad = fock1e.cal_grad_S(scgtos, S)
  if mode==1:  grad = fock1e.cal_grad_K(scgtos, S)
  if mode==2:  grad = fock1e.cal_grad_NA(scgtos, S, charges)
  if mode==3:  grad = cecp.cal_grad_ECP(scgtos, ecps, S)

  check_v=[0.0 for i in range(3)]
  for a in range( len(grad)/3 ):
    check_v[0] += math.sqrt(grad[a*3+0]*grad[a*3+0])
    check_v[1] += math.sqrt(grad[a*3+1]*grad[a*3+1])
    check_v[2] += math.sqrt(grad[a*3+2]*grad[a*3+2])

  ret=0
  if math.fabs(check_v[0] - exact_v[0])>1.0e-13 or \
     math.fabs(check_v[1] - exact_v[1])>1.0e-13 or \
     math.fabs(check_v[2] - exact_v[2])>1.0e-13:
    ret=1
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0:
      print " ----- error in td_fock1e_grad_base -----"
      print "exact_v[0] ",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]   
      print "exact_v[1] ",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]   
      print "exact_v[2] ",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[0]-check_v[2]   

  return ret



def td_fock1e_pbc_base(filename, exact_v, mode, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)

  ctrl_pbc = lotus_core_hpp.CTRL_PBC()
  T123 = lotus_core_hpp.get_vector_double(9)
  T123[0] =  1.0 
  T123[1] = -0.5 
  T123[2] =  0.0 
  T123[3] =  0.0 
  T123[4] = -1.5 
  T123[5] =  0.0 
  T123[6] =  0.0 
  T123[7] =  1.5 
  T123[8] =  1.0 
  max_Nc = lotus_core_hpp.get_vector_int(3)
  max_Nc[0] = 1
  max_Nc[1] = 2
  max_Nc[2] = 1
  max_Nt = lotus_core_hpp.get_vector_int(3)
  max_Nt[0] = 2
  max_Nt[1] = 2
  max_Nt[2] = 2
  ctrl_pbc.set_T123(T123)
  ctrl_pbc.set_max_Nc(max_Nc)
  ctrl_pbc.set_max_Nt(max_Nt)
#  ctrl_pbc.show()
  
  if mode_mat==0:
    fock1e = lotus_core_hpp.Fock1e()
    cecp   = lotus_core_hpp.ECP_matrix()
    S_PBC  = lotus_core_hpp.get_vector_dMatrix()
    M_PBC  = lotus_core_hpp.get_vector_dMatrix()
  elif mode_mat==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    cecp   = lotus_core_hpp.ECP_matrix_map()
    S_PBC  = lotus_core_hpp.get_vector_dMatrix_map()
    M_PBC  = lotus_core_hpp.get_vector_dMatrix_map()

  charges    = lotus_core_hpp.get_vector_Charge(2)
  charges[0] = lotus_core_hpp.Charge(1, 0.1, 0.2, 0.3, 0)
  charges[1] = lotus_core_hpp.Charge(2, 1.0, 1.1, 1.2, 1)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)

  if mode==0:  fock1e.cal_K_PBC(M_PBC, scgtos, ctrl_pbc)
  if mode==1:  fock1e.cal_NA_PBC(M_PBC, scgtos, charges, ctrl_pbc)
  if mode==2:  cecp.cal_ECP_PBC(M_PBC, scgtos, ecps, ctrl_pbc)

  N123_c = ctrl_pbc.get_N123_c()
  check_v = lotus_core_hpp.Util_PBC.cal_DV(S_PBC, M_PBC, N123_c)

  ret=0
  if math.fabs(check_v - exact_v)>1.0e-10:
    ret=1
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0:
      print " ----- error in td_fock1e_pbc_base -----"
      print "exact_v[0] ",exact_v,"check_v",check_v,"diff",exact_v-check_v   

  return ret






class td_fock1e(unittest.TestCase):
  
  def setUp(self):
    pass

  def test_k(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_k ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    check = td_fock1e_K_mat_base(filename,        15.209924623687, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_K_mat_base(filename,        15.209924623687, 1) 
    self.assertEqual(check, 0)

  def test_k_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_k_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    check = td_fock1e_K_mat_base(filename, 35.305943879327, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_K_mat_base(filename, 35.305943879327, 1) 
    self.assertEqual(check, 0)

  def test_na(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_na ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    check = td_fock1e_NA_mat_base(filename,        -14.6450507433424, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_NA_mat_base(filename,        -14.6450507433424, 1) 
    self.assertEqual(check, 0)

  def test_na_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_na_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    check = td_fock1e_NA_mat_base(filename, -35.5342442165267, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_NA_mat_base(filename, -35.5342442165267, 1) 
    self.assertEqual(check, 0)

  def test_ecp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_ecp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    check = td_fock1e_ECP_mat_base(filename,         0.668115104438203, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_ECP_mat_base(filename,         0.668115104438203, 1) 
    self.assertEqual(check, 0)

  def test_ecp_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_ecp_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    check = td_fock1e_ECP_mat_base(filename,  1.64195529039806, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_ECP_mat_base(filename,  1.64195529039806, 1) 
    self.assertEqual(check, 0)

  def test_grad_s(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_s ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v=[0.797858610874905,
             0.000000000000000,
             1.19156682184131]
    check = td_fock1e_grad_base(filename,        exact_v, 0, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename,        exact_v, 0, 1) 
    self.assertEqual(check, 0)

  def test_grad_s_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_s_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v=[1.76573600226019,
             0.00000000000000,
             2.55888122805177]
    check = td_fock1e_grad_base(filename, exact_v, 0, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename, exact_v, 0, 1) 
    self.assertEqual(check, 0)

  def test_grad_k(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_k ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v=[1.25963801096052,
             0.00000000000000,
             1.91082703634836]
    check = td_fock1e_grad_base(filename,        exact_v, 1, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename,        exact_v, 1, 1) 
    self.assertEqual(check, 0)

  def test_grad_k_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_k_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v=[3.20462307132361,
             0.00000000000000,
             4.67643987627959]
    check = td_fock1e_grad_base(filename, exact_v, 1, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename, exact_v, 1, 1) 
    self.assertEqual(check, 0)

  def test_grad_na(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_na ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v=[3.89035924054393,
             5.48287225486067,
             6.43108061233738]
    check = td_fock1e_grad_base(filename,        exact_v, 2, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename,        exact_v, 2, 1) 
    self.assertEqual(check, 0)

  def test_grad_na_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_na_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v=[ 9.73663174932849,
             12.7737287310335,
             16.1902064506491]
    check = td_fock1e_grad_base(filename, exact_v, 2, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename, exact_v, 2, 1) 
    self.assertEqual(check, 0)

  def test_grad_ecp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_ecp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v=[0.435739119392969,
             0.000000000000000,
             0.657556467625155]
    check = td_fock1e_grad_base(filename,        exact_v, 3, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename,        exact_v, 3, 1) 
    self.assertEqual(check, 0)

  def test_grad_ecp_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_ecp_plus_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v=[0.581764645449941,
             0.000000000000000,
             0.877934978556851]
    check = td_fock1e_grad_base(filename, exact_v, 3, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_grad_base(filename, exact_v, 3, 1) 
    self.assertEqual(check, 0)

  def test_pbc_k(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_k ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = 57.266002474008
    check = td_fock1e_pbc_base(filename,        exact_v, 0, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename,        exact_v, 0, 1) 
    self.assertEqual(check, 0)

  def test_pbc_k_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_k ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = 202.146619381262
    check = td_fock1e_pbc_base(filename, exact_v, 0, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename, exact_v, 0, 1) 
    self.assertEqual(check, 0)

  def test_pbc_na(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_na ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = -4711.62478666429
    check = td_fock1e_pbc_base(filename,        exact_v, 1, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename,        exact_v, 1, 1) 
    self.assertEqual(check, 0)

  def test_pbc_na_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_na ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = -15073.7573775886
    check = td_fock1e_pbc_base(filename, exact_v, 1, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename, exact_v, 1, 1) 
    self.assertEqual(check, 0)

  def test_pbc_ecp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_ecp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = 165.112814886142
    check = td_fock1e_pbc_base(filename,        exact_v, 2, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename,        exact_v, 2, 1) 
    self.assertEqual(check, 0)

  def test_pbc_ecp_plus_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_pbc_ecp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v =  644.769597094371
    check = td_fock1e_pbc_base(filename, exact_v, 2, 0) 
    self.assertEqual(check, 0)
    check = td_fock1e_pbc_base(filename, exact_v, 2, 1) 
    self.assertEqual(check, 0)


def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test1.py #####"
  suite      = unittest.TestLoader().loadTestsFromTestCase(td_fock1e)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  testRunner.run(suite)




if __name__ == "__main__":
  run_test()







import unittest
import lotus_core_hpp
import math
import os

def td_grad_base(filename, exact_v, mode_u, sel_functor, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)
  if mode_mat==0:
    fock1e   = lotus_core_hpp.Fock1e()
    S        = lotus_core_hpp.dMatrix()
    X_a      = lotus_core_hpp.dMatrix()
    X_b      = lotus_core_hpp.dMatrix()
  elif mode_mat==1:
    fock1e   = lotus_core_hpp.Fock1e_map()
    S        = lotus_core_hpp.dMatrix_map()
    X_a      = lotus_core_hpp.dMatrix_map()
    X_b      = lotus_core_hpp.dMatrix_map()

  lamda   = lotus_core_hpp.get_vector_double()
  lamda_b = lotus_core_hpp.get_vector_double()
  scf     = lotus_core_hpp.SCF()
  grad    = lotus_core_hpp.Gradient()
  moldata = lotus_core_hpp.MolData()
  moldata.set_basis_from_file(filename)

  if sel_functor==0: functor = lotus_core_hpp.HF_Functor()
  if sel_functor==1: functor = lotus_core_hpp.Slater_Functor()
  if sel_functor==2: functor = lotus_core_hpp.B88_Functor()
  if sel_functor==3: functor = lotus_core_hpp.SVWN_Functor()
  if sel_functor==4: functor = lotus_core_hpp.BLYP_Functor()
  if sel_functor==5: functor = lotus_core_hpp.B3LYP_Functor()
 

  if mode_u==0:
    scf.guess_h_core(X_a, lamda, moldata.scgtos, moldata.ecps)
    eri = lotus_core_hpp.ERI_incore()
    scf.r_scf_debug(X_a, lamda, moldata, eri, functor) 

    occ    = lotus_core_hpp.Util.cal_occ(scgtos, ecps, 0, 1)
    force  = grad.cal_force(moldata.scgtos, moldata.ecps, X_a, occ, lamda, functor)
  if mode_u==1:
    scf.guess_h_core(X_a, lamda, moldata.scgtos, moldata.ecps)
    lotus_core_hpp.mat_copy(X_b, X_a) 
    eri = lotus_core_hpp.ERI_incore()
    scf.u_scf_debug(X_a, X_b, lamda, lamda_b, moldata, eri, functor) 

    occ_a = lotus_core_hpp.get_vector_double()
    occ_b = lotus_core_hpp.get_vector_double()
    lotus_core_hpp.Util.cal_occ_ab(occ_a, occ_b, moldata.scgtos, moldata.ecps, moldata.mol_charge, moldata.spin)
    force = grad.cal_force_u(moldata.scgtos, moldata.ecps, X_a, X_b, occ_a, occ_b, lamda, lamda_b, functor)
 
 
  check_v  = lotus_core_hpp.get_vector_double(3)
  check_v[0]=0.0
  check_v[1]=0.0
  check_v[2]=0.0
  for a in range( len(force)/3 ):
    check_v[0] += math.sqrt(force[a*3+0]*force[a*3+0]) 
    check_v[1] += math.sqrt(force[a*3+1]*force[a*3+1]) 
    check_v[2] += math.sqrt(force[a*3+2]*force[a*3+2]) 
    print "a=",a,"  force ",force[a*3+0],force[a*3+1],force[a*3+2],"check_v",check_v[0],check_v[1],check_v[2]

  ret=0
  process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  abs_error=1.0e-7
  if abs( check_v[0]-exact_v[0])>abs_error or abs(check_v[1]-exact_v[1])>abs_error or \
     abs( check_v[2]-exact_v[2])>abs_error:
    ret=1
    if process_num==0:
      print " ----- in td_grad_base -----"
      print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
      print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
      print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]

  return ret





class td_grad(unittest.TestCase):
  
  def setUp(self):
    pass

  def fun_grad_sub(self, filename, exact_v, sel_functor):
    check = td_grad_base(filename, exact_v, 0, sel_functor, 0) 
    self.assertEqual(check, 0)
    check = td_grad_base(filename, exact_v, 0, sel_functor, 1) 
    self.assertEqual(check, 0)
    check = td_grad_base(filename, exact_v, 1, sel_functor, 0) 
    self.assertEqual(check, 0)
    check = td_grad_base(filename, exact_v, 1, sel_functor, 1) 
    self.assertEqual(check, 0)
 
 
  def test_grad_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_hf ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 0.3493498502009253,
                0.0000000000000000,
                0.4544301004312827]
    self.fun_grad_sub(filename, exact_v, 0)

  def test_grad_hf_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_hf_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 0.3427749363440474,
                0.0000000000000000,
                0.4309958979772428]
    self.fun_grad_sub(filename_d, exact_v, 0)

  def test_grad_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 2.405923838182816e-01,
                0.0000000000000000,
                2.895087230952800e-01]
    self.fun_grad_sub(filename, exact_v, 3)

  def test_grad_svwn_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_svwn_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.448015576864108e-01,
                0.0000000000000000,
                2.818207168152005e-01]
    self.fun_grad_sub(filename_d, exact_v, 3)

  def test_grad_blyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_blyp_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.220973708021526e-01,
                0.0000000000000000,
                2.539773201615168e-01]
    self.fun_grad_sub(filename_d, exact_v, 4)

  def test_grad_b3lyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_grad_b3lyp_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.516472744362920e-01,
                0.0000000000000000,
                2.974812046657984e-01]
    self.fun_grad_sub(filename_d, exact_v, 5)




def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test7.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_grad)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  unittest.TextTestRunner().run(suite)


if __name__ == "__main__":
  run_test()






import os
import sys
import copy



class In_Scgto:
  def __init__(self):
    self.atom_no    = None
    self.atom_kind  = None
    self.Rxyz       = None
    self.shell_type = None 
    self.num_pgto   = 0
    self.g = []
    self.d = []

  def show(self):
    print "   ----- Scgto ",self.shell_type,self.num_pgto,"  ----"
    for i in range(self.num_pgto):
      print "      i=",i," g, d ", self.g[i], self.d[i]

  def show2(self):
    print "     ----- Scgto: atom_kind ",self.atom_kind,"atom_no",self.atom_no,
    print " Rxyz ",self.Rxyz[0],self.Rxyz[1],self.Rxyz[2]
    print "      shell_type, num_pgto ", self.shell_type, self.num_pgto,"  ----"
    for i in range(self.num_pgto):
      print "        i=",i," g, d ", self.g[i], self.d[i]


class IN_ECP:
  def __init__(self):
    self.atom_no    = None
    self.atom_kind  = None 
    self.Rxyz       = None 
    self.shell_type = None 
    self.core_ele   = 0
    self.num_pecp   = 0
    self.g          = []
    self.k          = []
    self.d          = []

  def show(self):
    print "  ----- IN_ECP  shell_type, core_ele, num_pecp ",self.shell_type,core_ele,self.num_pecp," -----"
    for i in range(self.num_pecp):
      print "      i=",i," g, ,k, d ", self.g[i],"  ", self.k[i],"  ", self.d[i]

  def show2(self):
    print "  ----- IN_ECP: atom_kind ",self.atom_kind,"atom_no",self.atom_no,
    print " Rxyz ",self.Rxyz[0],self.Rxyz[1],self.Rxyz[2]
    print "    shell_type, num_pecp,core_ele ",self.shell_type, self.num_pecp, self.core_ele
    for i in range(self.num_pecp):
      print "      i=",i," g, ,k, d ", self.g[i],"  ", self.k[i],"  ", self.d[i]



class ReadInput:

  class Str_info:
    def __init__(self):
      self.atoms=[]

    def get_nth_atom_kind(self,nth):
      return copy.deepcopy(self.atoms[nth][0])

    def get_nth_Rxyz(self,nth):
      ret = []
      ret.append(self.atoms[nth][1])
      ret.append(self.atoms[nth][2])
      ret.append(self.atoms[nth][3])
      return ret

    def show(self):
      print self.atoms
      for i in range( len(self.atoms) ):
        Rxyz = self.get_nth_Rxyz(i)
        print "   Atom[",i,"]:  atom_kind ",self.get_nth_atom_kind(i),"  Rxyz  ",Rxyz[0],Rxyz[1],Rxyz[2] 
  

  @staticmethod
  def read_str_info_from_instring(instring):
    ret_str_info = ReadInput.Str_info()
    a_to_au = 1.889725989 
    lines = instring.lower().split("\n")
    for li in lines:
      ws = li.split()
      if len(ws)!=0:
        if ws[0].lower()=="atom:":
          a_data = []
          a_data.append(ws[1])
          a_data.append( float(ws[2])*a_to_au )
          a_data.append( float(ws[3])*a_to_au )
          a_data.append( float(ws[4])*a_to_au )
          ret_str_info.atoms.append(a_data)
    return ret_str_info


  @staticmethod
  def read_str_info_from_infile(filename):
    instring = ""
    if os.path.isfile(filename)==True:  instring = open(filename).read()
    return read_str_info_from_instring(instring)



  @staticmethod
  def read_keyvalue_from_instring(key,instring):
    ret=None
    lines = instring.lower().split("\n")
    for li in lines:
      ws = li.split()
      if len(ws)!=0:
        if ws[0].lower()==key:
          ret=ws[1]
          return ret


  @staticmethod
  def get_lines_from_basis_file(basis_name):
    filename_bare = basis_name + ".basis"
    if os.path.isfile(filename_bare)==True:  btext = open(filename_bare).read()
    #
    filename_cwd = os.getcwd() + "/" + basis_name + ".basis"
    if os.path.isfile(filename_cwd)==True:  btext = open(filename_cwd).read()
    #
    filename_basis_DATA = os.path.dirname(__file__) + "/basis_DATA/" + basis_name + ".basis"
    if os.path.isfile(filename_basis_DATA)==True:  btext = open(filename_basis_DATA).read()
    # 
    return btext.lower().split("\n")


  @staticmethod
  def get_basis_lines_for_atom(sel_atom, basis_lines):
    i = 0
    for li in basis_lines:
      words     = li.split()
      pre_words = []
      if i!=0: pre_words = basis_lines[i-1].split()
      if len(words)!=0 and len(pre_words)!=0:
        if words[0] == sel_atom.lower() and pre_words[0] == "****":
          ret_lines = []
          j=0
          while len(basis_lines[i+j].split())!=0:
            ws = basis_lines[i+j].split()
            if ws[0]=="****":  return ret_lines
            else:              ret_lines.append(basis_lines[i+j])
            j=j+1
          return ret_lines
      i=i+1

  @staticmethod
  def get_ecp_lines_for_atom(sel_atom, basis_lines):
    sel_ecp = sel_atom.lower() + "-ecp"
    it = iter(basis_lines)
    ret_lines = []
    for li in it:
      words = li.split()
      if len(words)!=0:
        if words[0].lower()==sel_ecp:
          ret_lines.append(li)

          for li2 in it:
            if li2.lower().find("ecp")==-1:
              ret_lines.append(li2)
            else:
              del ret_lines[ len(ret_lines)-1 ]
              return ret_lines 


  class BasisData:
    def __init__(self):
      self.atom   = None
      self.scgtos = []

    def show(self):
      print "----- BasisData ",self.atom,len(self.scgtos),"-----"
      for i in range(len(self.scgtos)):
        print "   ///// i=",i,"//////"
        self.scgtos[i].show()


  class ECP_DATA:
    def __init__(self):
      self.atom_kind   = None 
      self.ecps        = []
      self.num_ecps    = 0
      self.core_ele    = 0

    def show(self):
      print "----- ECP_DATA ",self.atom_kind,len(self.ecps),"-----"
      for i in range(len(self.ecps)):
        print "   ///// i=",i,"//////"
        self.ecps[i].show()      

  @staticmethod 
  def shell_type2int(st):
    if st=="s":  return 0
    if st=="p":  return 1
    if st=="d":  return 2
    if st=="f":  return 3
    if st=="g":  return 4
    if st=="h":  return 5
    if st=="i":  return 6


  @staticmethod
  def get_atom_basis_data_from_lines(basis_lines):
    it = iter(basis_lines)


    words = it.next().split()
    ret_basis_data= ReadInput.BasisData()
    ret_basis_data.atom = words[0]
    for li in it:
      words = li.split()
      if words[0]=="sp":
        data_s = In_Scgto()
        data_p = In_Scgto()
        data_s.shell_type=ReadInput.shell_type2int("s")
        data_p.shell_type=ReadInput.shell_type2int("p")
        data_s.num_pgto = int(words[1])
        data_p.num_pgto = int(words[1])
        for j in range( int(words[1]) ):
          li = it.next()
          w_gd = li.split()
          data_s.g.append( float(w_gd[0]) )
          data_s.d.append( float(w_gd[1]) )
          data_p.g.append( float(w_gd[0]) )
          data_p.d.append( float(w_gd[2]) )
        ret_basis_data.scgtos.append(data_s)
        ret_basis_data.scgtos.append(data_p)
      else:
        data = In_Scgto()
        data.shell_type = ReadInput.shell_type2int(words[0])
        data.num_pgto   = int(words[1])
        for j in range( int(words[1]) ):
          li = it.next()
          w_gd = li.split()
          data.g.append( float(w_gd[0]) )
          data.d.append( float(w_gd[1]) )
        ret_basis_data.scgtos.append(data)
    return ret_basis_data    

  @staticmethod
  def get_ecp_shell_type(string):
    if string.find("-")==-1:
      return -1
    if string.find("s-")==0:
      return 0
    if string.find("p-")==0:
      return 1
    if string.find("d-")==0:
      return 2
    if string.find("f-")==0:
      return 3
    if string.find("g-")==0:
      return 4
    if string.find("h-")==0:
      return 5
    if string.find("i-")==0:
      return 6
    

  @staticmethod
  def get_atom_ecp_data_from_lines(sel_atom, ecp_lines):
    it = iter(ecp_lines)
    ret_ecp_data = ReadInput.ECP_DATA()
    ret_ecp_data.atom_kind = sel_atom
    words = it.next().split()
    ret_ecp_data.num_ecps = int(words[1])
    core_ele = int(words[2])
    flag_ecp_L = 0

    for li in it:
      ecp = IN_ECP()
      ecp.core_ele = core_ele
      words = li.split()
      if flag_ecp_L==0:
        ecp.shell_type = -1
        flag_ecp_L=1
      else:
        ecp.shell_type = ReadInput.get_ecp_shell_type(words[0])
      ecp.num_pecp   = int(it.next().split()[0])
      for i in range( ecp.num_pecp ):
        ws_kgd = it.next().split()
        ecp.k.append( float(ws_kgd[0]) )
        ecp.d.append( float(ws_kgd[1]) )
        ecp.g.append( float(ws_kgd[2]) )
      ret_ecp_data.ecps.append(ecp)
    return ret_ecp_data


  @staticmethod
  def get_atom_basis(atom_name, blines):
    alines = ReadInput.get_basis_lines_for_atom(atom_name, blines)
    if alines!=None:
      return ReadInput.get_atom_basis_data_from_lines(alines)  
    else:
      print " cannot find atom,",atom_name," in basis file "


  @staticmethod
  def get_atom_ecp(atom_name, blines):
    ecp_lines = ReadInput.get_ecp_lines_for_atom(atom_name, blines)
    if ecp_lines!=None:
      return ReadInput.get_atom_ecp_data_from_lines(atom_name, ecp_lines)
    else:
      ecp_data = []
      return ecp_data
    


  @staticmethod
  def get_scgtos_from_lines(str_info, blines):
    set_atoms = set([])
    for li_atm in str_info.atoms:
      set_atoms.add(li_atm[0])
    #
    dic_basis = {}
    for atm in set_atoms:
      abasis = ReadInput.get_atom_basis(atm, blines)
      dic_basis[atm] = abasis
    #
    ret_scgtos = []
    for i in range( len(str_info.atoms) ):
      atm        = str_info.get_nth_atom_kind(i)
      Rxyz       = str_info.get_nth_Rxyz(i)
      #
      basis_data = dic_basis[atm]
      for j in range( len(basis_data.scgtos) ): 
        scgto           = copy.deepcopy( basis_data.scgtos[j] )
        scgto.atom_no   = i
        scgto.atom_kind = atm 
        scgto.Rxyz      = Rxyz
        ret_scgtos.append(scgto)
      
    return ret_scgtos


  @staticmethod
  def get_ecps_from_lines(str_info, blines):
    set_atoms = set([])
    for li_atm in str_info.atoms:
      set_atoms.add(li_atm[0])
    #
    dic_ecp = {}
    for atm in set_atoms:
      aecp           = ReadInput.get_atom_ecp(atm, blines)
      dic_ecp[atm]   = aecp
    #
    ret_ecps = []
    for i in range( len(str_info.atoms) ):
      atm        = str_info.get_nth_atom_kind(i)
      Rxyz       = str_info.get_nth_Rxyz(i)
      #
      ecp_data   = dic_ecp[atm]
      if ecp_data!=[]:
        for j in range( len(ecp_data.ecps) ): 
          ecp            = copy.deepcopy( ecp_data.ecps[j] )
          ecp.atom_no    = i
          ecp.atom_kind  = atm
          ecp.Rxyz       = Rxyz 
          ret_ecps.append(ecp)
    return ret_ecps




  @staticmethod 
  def get_atom_Z(atom_kind):
    map_Z = { "h":1,                                                                                                                                 "he":2,
             "li":3, "be":4,                                                                                  "b":5,  "c":6,  "n":7,  "o":8,  "f":9, "ne":10,
             "na":11,"mg":12,                                                                                "al":13,"si":14, "p":15, "s":16,"cl":17,"ar":18,
              "k":19,"ca":20,"sc":21,"ti":22, "v":23,"cr":24,"mn":25,"fe":26,"co":27,"ni":28,"cu":29,"zn":30,"ga":31,"ge":32,"as":33,"se":34,"br":35,"kr":36,
             "rb":37,"sr":38, "y":29,"zr":40,"nb":41,"mo":42,"tc":43,"ru":44,"rh":45,"pd":46,"ag":47,"cd":48,"in":49,"sn":50,"sb":51,"te":52, "i":53,"xe":54,
             "cs":55,"ba":56,        "hf":72,"ta":73, "w":74,"re":75,"os":76,"ir":77,"pt":78,"au":79,"hg":80,"ti":81,"pb":82,"bi":83,"po":84,"at":85,"rn":86,
             "fr":87,"ra":88,        "rf":104,"db":105,"sg":106,"bh":107,"hs":108,"mt":109,"ds":110,

             "la":57,"ce":58,"pr":59,"nd":60,"pm":61,"sm":62,"eu":63,"gd":64,"tb":65,"dy":66,"ho":67,"er":68,"tm":69,"yb":70,"lu":71,
             "ac":89,"th":90,"pa":91, "u":92,"np":93,"pu":94,"am":95,"cm":96,"bk":97,"cf":98,"es":99,"fm":100,"md":101,"no":102,"lr":103
            }   
    return map_Z[ atom_kind.lower() ]
 
 

  @staticmethod
  def get_qm_string(atoms, scgtos, ecps=[]):
    N_scgtos = len(scgtos)
    N_ecps   = 0
    N_atoms  = scgtos[N_scgtos-1].atom_no+1
    if ecps!=[]:
      N_ecps = len(ecps)
    #
    ret_string = ""
    ret_string += "__________LOTUS_MOLECULE_INPUT_FILE__________\n\n"
    ret_string += "N_scgtos__N_ecps_N_atoms\n"
    ret_string += str(N_scgtos) + "  " + str(N_ecps) + "  " + str(N_atoms) + "\n\n"
    #
    ret_string += "__________MOLECULAR_COORDINATE_________\n"
    for i in range(len(atoms)):
      atom_kind = atoms[i][0].lower()
      x = atoms[i][1]
      y = atoms[i][2]
      z = atoms[i][3]
      atom_z = ReadInput.get_atom_Z(atom_kind)
      ret_string += "%4s  %4d  atom_z  %4d   xyz  %15f  %15f  %15f \n" % (atom_kind,atom_z,i,x,y,z)
    ret_string += "\n"
    
    ret_string += "__________SCGTOS_DATA__________\n"
    for i in range(N_scgtos):
      atom_kind = atoms[scgtos[i].atom_no][0].lower()
      atom_z    = ReadInput.get_atom_Z(atom_kind)
      x         = scgtos[i].Rxyz[0] 
      y         = scgtos[i].Rxyz[1] 
      z         = scgtos[i].Rxyz[2] 
      ret_string += "scgto: no " + str(i) + " shell_type " + str(scgtos[i].shell_type) + " atom_no " + str(scgtos[i].atom_no)
      ret_string += " atom_kind " + str(atom_kind) + " atom_z " + str(atom_z) + "   xyz   "+str(x)+"  "+str(y)+"  "+str(z)+"\n"
      ret_string += str(scgtos[i].num_pgto) + "\n"
      for j in range(scgtos[i].num_pgto):
        ret_string += "   %15f   %15f  \n" % (scgtos[i].g[j],scgtos[i].d[j])
    ret_string += "\n"
    # 
    ret_string += "__________ECP_DATA__________\n"
    for i in range(N_ecps):
      atom_no    = ecps[i].atom_no
      core_ele   = ecps[i].core_ele
      shell_type = ecps[i].shell_type
      atom_kind  = atoms[atom_no][0].lower()
      atom_z     = ReadInput.get_atom_Z(atom_kind)
      x          = ecps[i].Rxyz[0]
      y          = ecps[i].Rxyz[1]
      z          = ecps[i].Rxyz[2]
      ret_string += "ecps: no "+str(i)+" core_ele "+str(core_ele)+" shell_type "+str(shell_type)+" atom_no "+str(atom_no)
      ret_string += " atom_kind "+atom_kind+" atom_z "+str(atom_z)+"    xyz   "+str(x)+"  "+str(y)+"  "+str(z)+"\n"
      ret_string += str(ecps[i].num_pecp) + "\n"
      for j in range(ecps[i].num_pecp):
        ret_string += "  %15f  %4d  %15f " % (ecps[i].g[j], ecps[i].k[j], ecps[i].d[j]) + "\n"
    ret_string += "\n"
     
    return ret_string 

  @staticmethod 
  def get_qm_string_from_infile(infile):
    basis_name     = ReadInput.read_keyvalue_from_infile("basis:",    infile)
    basis_lines    = ReadInput.get_lines_from_basis_file(basis_name)
    str_info       = ReadInput.read_str_info_from_infile(infile)
    atoms     = str_info.atoms
    scgtos    = ReadInput.get_scgtos_from_lines(str_info, basis_lines)
    ecps      = ReadInput.get_ecps_from_lines(str_info, basis_lines)
    return ReadInput.get_qm_string(atoms, scgtos, ecps)   
 








import lotus_core_hpp



class Wrapper4Lapack:

  def __init__(self):
    import platform
    if platform.system()=='Darwin':
      self.default_path="/usr/lib/liblapack.dylib"
    else:
      self.default_path="/usr/lib/liblapack.so"


  def get_liblapack(self, in_path):
    import ctypes

    path=in_path
    if path=="default":
      path=self.default_path
    if path=="autofind":
      import ctypes.util
      path=ctypes.util.find_library("lapack")
      if path==None:
        print "ERROR: fail in auto-find lapack library"
        quit()

    try:
      liblapack = ctypes.CDLL(path)
    except:
      print " ERROR: wrapper4lapack.inveser() can not find shared-type lapack library on the path:",path
      print "        please give appropriate library path and environment variables for lapack."
      quit()
    return liblapack
                             

  def inverse(self, m, path="default"):
    import ctypes
    liblapack = self.get_liblapack(path)

    I = m.get_I()
    J = m.get_J()
    lwork=4*I
    c_I     = ctypes.byref( ctypes.c_int(I) )
    c_J     = ctypes.byref( ctypes.c_int(J) )
    c_lda   = ctypes.byref( ctypes.c_int(I) )
    c_lwork = ctypes.byref( ctypes.c_int(lwork) )
    c_ipiv  = ctypes.byref( (ctypes.c_int*I)() )
    c_work  = ctypes.byref( (ctypes.c_double*lwork)() )
    c_info  = ctypes.c_int(0)
    c_mat   = (ctypes.c_double*(I*J))()
    for i in range(I):
      for j in range(J):
        c_mat[j*I+i]=m.get_value(i,j)

    liblapack.dgetrf_(c_I, c_J, ctypes.byref(c_mat), c_lda, c_ipiv, ctypes.byref(c_info))
    if c_info.value!=0:
      print "ERROR info=",c_info.value," in dgetrf of wrapper4lapack.inverse() "
      quit()

    liblapack.dgetri_(c_I, ctypes.byref(c_mat), c_lda, c_ipiv, \
                      c_work, c_lwork, ctypes.byref(c_info))
    if c_info.value!=0:
      print "ERROR info=",c_info.value," in dgetri of wrapper4lapack.inverse() "
      quit()

    for i in range(I):
      for j in range(J):
        tmp_v = c_mat[j*I+i]
        m.set_value(i,j, tmp_v)


  def dgesv(self, A, X, B, path="default"):
    import ctypes
    liblapack = self.get_liblapack(path)

    N    = A.get_I()
    nrhs = B.get_J()
    c_N     = ctypes.byref( ctypes.c_int(N) )
    c_nrhs  = ctypes.byref( ctypes.c_int(nrhs) )
    c_lda   = ctypes.byref( ctypes.c_int(N) )
    c_ldb   = ctypes.byref( ctypes.c_int(N) )
    c_ipiv  = ctypes.byref( (ctypes.c_int*N)() )
    c_info  = ctypes.c_int(0)
    c_mat_A = (ctypes.c_double*(N*N))()
    c_mat_B = (ctypes.c_double*(N*nrhs))()

    for i in range(N):
      for j in range(N):
        c_mat_A[j*N+i]=A.get_value(i,j)

    for i in range(N):
      for j in range(nrhs):
        c_mat_B[j*N+i]=B.get_value(i,j)

    liblapack.dgesv_(c_N, c_nrhs, ctypes.byref(c_mat_A), c_lda, c_ipiv, \
                     ctypes.byref(c_mat_B), c_ldb, ctypes.byref(c_info))

    if c_info.value!=0:
      print "ERROR info=",c_info.value," in dgesv "
      quit()

    X.set_IJ(N, nrhs)
    for i in range(N):
      for j in range(nrhs):
        X.set_value(i,j, c_mat_B[j*N+i])

    return c_info.value


  # A X = lamda X , calculate eigen value and eigen vector
  def dsyev(self, A, X, lamda, path="default"):
    import ctypes
    liblapack = self.get_liblapack(path)

    N     = A.get_I()
    lwork = N*4 
    c_jobz  = ctypes.byref( ctypes.c_char('V') )
    c_uplo  = ctypes.byref( ctypes.c_char('U') )
    c_N     = ctypes.byref( ctypes.c_int(N) )
    c_lda   = ctypes.byref( ctypes.c_int(N) )
    c_lwork = ctypes.byref( ctypes.c_int(lwork) )
    c_ipiv  = ctypes.byref( (ctypes.c_int*N)() )
    c_work  = ctypes.byref( (ctypes.c_double*lwork)() )

    c_info  = ctypes.c_int(0)
    c_lamda = (ctypes.c_double*(N))()
    c_mat_X = (ctypes.c_double*(N*N))()

    for i in range(N):
      for j in range(N):
        c_mat_X[j*N+i]=A.get_value(i,j)

    liblapack.dsyev_(c_jobz, c_uplo, c_N, ctypes.byref(c_mat_X), c_lda, c_lamda, \
                     c_work, c_lwork, ctypes.byref(c_info))
    if c_info.value!=0:
      print "ERROR info=",c_info.value," in lapack_dsyev "
      quit()

    X.set_IJ(N, N)
    lotus_core_hpp.vector_clear(lamda)
    for i in range(N):
      lamda.append(c_lamda[i])
      for j in range(N):
        X.set_value(i, j, c_mat_X[j*N+i])

    return c_info.value


  # A X = lamda S X , calcualte eigen value and eigen vector
  def dsygv(self, A, S, X, lamda, path="default"):
    import ctypes
    liblapack = self.get_liblapack(path)

    N     = A.get_I()
    lwork = N*4 
    c_itype = ctypes.byref( ctypes.c_int(1) )
    c_jobz  = ctypes.byref( ctypes.c_char('V') )
    c_uplo  = ctypes.byref( ctypes.c_char('U') )
    c_N     = ctypes.byref( ctypes.c_int(N) )
    c_lda   = ctypes.byref( ctypes.c_int(N) )
    c_ldb   = ctypes.byref( ctypes.c_int(N) )
    c_lwork = ctypes.byref( ctypes.c_int(lwork) )
    c_ipiv  = ctypes.byref( (ctypes.c_int*N)() )
    c_work  = ctypes.byref( (ctypes.c_double*lwork)() )

    c_info  = ctypes.c_int(0)
    c_lamda = (ctypes.c_double*(N))()
    c_mat_X = (ctypes.c_double*(N*N))()
    c_mat_S = (ctypes.c_double*(N*N))()

    for i in range(N):
      for j in range(N):
        c_mat_X[j*N+i]=A.get_value(i,j)
        c_mat_S[j*N+i]=S.get_value(i,j)

    liblapack.dsygv_(c_itype, c_jobz, c_uplo, c_N, ctypes.byref(c_mat_X), c_lda, \
                     ctypes.byref(c_mat_S), c_ldb, c_lamda, c_work, c_lwork, ctypes.byref(c_info))
                    
    if c_info.value!=0:
      print "ERROR info=",c_info.value," in lapack_dsygv "
      quit()

    X.set_IJ(N, N)
    lotus_core_hpp.vector_clear(lamda)
    for i in range(N):
      lamda.append(c_lamda[i])
      for j in range(N):
        X.set_value(i, j, c_mat_X[j*N+i])

    return c_info.value









import unittest
import lotus_core_hpp
import lotus_core
import math
import os


def check_M(A, X, lamda):
  N_cgtos = len(lamda)
  Ax = lotus_core_hpp.get_vector_double(N_cgtos)

  abs_error=0.0
  for q in range(N_cgtos):
    for i in range(N_cgtos):
      tmp_v_i = 0.0
      for j in range(N_cgtos):
        tmp_v_i += A.get_value(i,j)*X.get_value(j,q)
      Ax[i]=tmp_v_i

    for i in range(N_cgtos):
      tmp_c = Ax[i] - lamda[q]*X.get_value(i,q)
      abs_error += abs(tmp_c)
  
  return abs_error


def check_M2(A, S, X, lamda, SX):
  N_cgtos = len(lamda)
  Ax = []
  [Ax.append(0.0) for i in range(N_cgtos)]
  SX.set_IJ(N_cgtos, N_cgtos)
  lotus_core_hpp.mat_mul(SX, S, X)

  abs_error=0.0
  for q in range(N_cgtos):
    for i in range(N_cgtos):
      tmp_v_i = 0.0
      for j in range(N_cgtos):
        tmp_v_i += A.get_value(i,j)*X.get_value(j,q)
      Ax[i]=tmp_v_i

    for i in range(N_cgtos):
      tmp_c = Ax[i] - lamda[q]*SX.get_value(i,q)
      abs_error += abs(tmp_c)
  
  return abs_error


def td_matrix_base(filename, exact_v, mode_mat):
  lamda = lotus_core_hpp.get_vector_double()

  if mode_mat==0:
    fock1e  = lotus_core_hpp.Fock1e()
    lotus   = lotus_core.create_lotus("dMatrix")
    S       = lotus_core_hpp.dMatrix()
    X       = lotus_core_hpp.dMatrix()
    SX      = lotus_core_hpp.dMatrix()
    H_core  = lotus_core_hpp.dMatrix()
    zS      = lotus_core_hpp.zMatrix()
    zX      = lotus_core_hpp.zMatrix()
    zH_core = lotus_core_hpp.zMatrix()
    zSX     = lotus_core_hpp.zMatrix()
  elif mode_mat==1:
    fock1e  = lotus_core_hpp.Fock1e_map()
    lotus   = lotus_core.create_lotus("dMatrix_map")
    S       = lotus_core_hpp.dMatrix_map()
    X       = lotus_core_hpp.dMatrix_map()
    SX      = lotus_core_hpp.dMatrix_map()
    H_core  = lotus_core_hpp.dMatrix_map()
    zS      = lotus_core_hpp.zMatrix_map()
    zX      = lotus_core_hpp.zMatrix_map()
    zH_core = lotus_core_hpp.zMatrix_map()
    zSX     = lotus_core_hpp.zMatrix_map()
  
  
  check_v = lotus_core_hpp.get_vector_double(10)
  process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  if process_num==0: print type(S)

  lotus.read_in_file(filename)
  
  lotus.cal_matrix("S", S)
  lotus.cal_matrix("H_core", H_core)

  lotus_core_hpp.mat_copy(zS, S) 
  lotus_core_hpp.mat_copy(zH_core, H_core) 

  lotus_core_hpp.cal_eigen(H_core, X, lamda)
  check_v[0] = check_M(H_core, X, lamda)

  lotus_core_hpp.cal_z_eigen(zH_core, zX, lamda)
  check_v[1] = check_M(H_core, X, lamda)

  lotus_core_hpp.cal_eigen(H_core, S, X, lamda)
  check_v[2] = check_M2(H_core, S, X, lamda, SX)

  lotus_core_hpp.cal_z_eigen(zH_core, zS, zX, lamda)
  check_v[3] = check_M2(zH_core, zS, zX, lamda, zSX)

  ret=0
  abs_error=5.0e-11
  if abs( check_v[0]-exact_v[0])>abs_error or abs(check_v[1]-exact_v[1])>abs_error or \
     abs( check_v[0]-exact_v[0])>abs_error or abs(check_v[1]-exact_v[1])>abs_error:
    ret=1
    print " ----- in td_matrix_base -----"
    print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
    print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
    print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]
    print "   exact_v[3]",exact_v[3],"check_v[3]",check_v[3],"diff",exact_v[3]-check_v[3]

  return ret



class td_matrix(unittest.TestCase):
  
  def setUp(self):
    pass
  
  def test_matrix(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_matrix ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_6-31gs.in"
    exact_v = lotus_core_hpp.get_vector_double(10)
    check = td_matrix_base(filename, exact_v, 0)
    self.assertEqual(check, 0)
    check = td_matrix_base(filename, exact_v, 1)
    self.assertEqual(check, 0)


def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test5.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_matrix)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  unittest.TextTestRunner().run(suite)


if __name__ == "__main__":
  run_test()





import lotus_core_hpp

from mixin_core          import *
from read_basis_DATA     import *
from impl_wrapper4lapack import *

def vector2list(vec):
  N = len(vec)
  li = []
  for i in range(N):
    li.append(vec[i])
  return li

def list2vector_int(li):
  N = len(li)
  vec = lotus_core_hpp.get_vector_int(N)
  for i in range(N):
    vec[i] = li[i]
  return vec

def list2vector_double(li):
  N = len(li)
  vec = lotus_core_hpp.get_vector_double(N)
  for i in range(N):
    vec[i] = li[i]
  return vec


 

 




import unittest
import lotus_core_hpp
import lotus_core
import math
import os



def td_scf_base(filename, exact_v, mode, sel_eri, sel_functor, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)

  if mode_mat==0:
    fock1e   = lotus_core_hpp.Fock1e()
    S        = lotus_core_hpp.dMatrix()
    D        = lotus_core_hpp.dMatrix()
    checkM_a = lotus_core_hpp.dMatrix()
    checkM_b = lotus_core_hpp.dMatrix()
  elif mode_mat==1:
    fock1e   = lotus_core_hpp.Fock1e_map()
    S        = lotus_core_hpp.dMatrix_map()
    D        = lotus_core_hpp.dMatrix_map()
    checkM_a = lotus_core_hpp.dMatrix_map()
    checkM_b = lotus_core_hpp.dMatrix_map()

  lamda   = lotus_core_hpp.get_vector_double()
  lamda_b = lotus_core_hpp.get_vector_double()
  scf     = lotus_core_hpp.SCF()

  if sel_eri==0: eri = lotus_core_hpp.ERI_direct() 
  if sel_eri==1: eri = lotus_core_hpp.ERI_file() 
  if sel_eri==2: eri = lotus_core_hpp.ERI_incore() 

  if sel_functor==0: functor = lotus_core_hpp.HF_Functor()
  if sel_functor==1: functor = lotus_core_hpp.Slater_Functor()
  if sel_functor==2: functor = lotus_core_hpp.B88_Functor()
  if sel_functor==3: functor = lotus_core_hpp.SVWN_Functor()
  if sel_functor==4: functor = lotus_core_hpp.BLYP_Functor()
  if sel_functor==5: functor = lotus_core_hpp.B3LYP_Functor()

  fock1e.cal_S(S, scgtos)

  process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  if process_num==0: print "type(S)", type(S)
  abs_error=1.0e-9

  if mode==0:  # cal_h_core
    scf.cal_h_core(checkM_a, scgtos, ecps)
    check_a = lotus_core_hpp.Util.cal_DV(S, checkM_a)
    check_b = 0.0
  if mode==1:  # guess_h_core
    scf.guess_h_core(checkM_a, lamda, scgtos, ecps)
    occ = lotus_core_hpp.Util.cal_occ(scgtos, ecps, 0, 1)
    check_a = 0.0
    for i in range( len(lamda) ): check_a+=math.sqrt(lamda[i]*lamda[i])
    lotus_core_hpp.Util.cal_D(D, checkM_a, occ)
    check_b = lotus_core_hpp.Util.cal_DV(S, D)
  if mode==2: # cal_Fock_sub
    scf.prepare_eri(eri, scgtos) 
    scf.cal_Fock_sub(checkM_a, S, scgtos, eri, functor)
    check_a = lotus_core_hpp.Util.cal_DV(S, checkM_a)
    check_b = check_a
  if mode==3: # r_scf
    moldata = lotus_core_hpp.MolData()
    moldata.set_basis_from_file(filename)
    scf.guess_h_core(checkM_a, lamda, moldata.scgtos, moldata.ecps)
    scf.r_scf_debug(checkM_a, lamda, moldata, eri, functor)
    abs_error=1.0e-5
    check_a = scf.get_ene_total()
    check_b = 0.0
    for i in range( len(lamda) ):  check_b += math.sqrt( lamda[i]*lamda[i] )
  if mode==4: # cal_Fock_sub_u
    scf.prepare_eri(eri, scgtos) 
    scf.cal_Fock_sub_u(checkM_a, checkM_b, S, S, scgtos, eri, functor)
    check_a = lotus_core_hpp.Util.cal_DV(S, checkM_a)
    check_b = lotus_core_hpp.Util.cal_DV(S, checkM_b)
  if mode==5: # u_scf
    moldata = lotus_core_hpp.MolData()
    moldata.set_basis_from_file(filename)
    scf.guess_h_core(checkM_a, lamda, moldata.scgtos, moldata.ecps)
    lotus_core_hpp.mat_copy(checkM_b, checkM_a)
    scf.u_scf_debug(checkM_a, checkM_b, lamda, lamda_b, moldata, eri, functor)
    abs_error=1.0e-5
    check_a = scf.get_ene_total()
    check_b = 0.0
    for i in range( len(lamda) ):  check_b += math.sqrt( lamda[i]*lamda[i] )
  if mode==6: # r_rsc, eddis
    moldata = lotus_core_hpp.MolData()
    moldata.set_basis_from_file(filename)
    scf.guess_h_core(checkM_a, lamda, moldata.scgtos, moldata.ecps)
    scf.do_ediis()
    scf.r_scf_debug(checkM_a, lamda, moldata, eri, functor)
    abs_error=1.0e-5
    check_a = scf.get_ene_total()
    check_b = 0.0
    for i in range( len(lamda) ):  check_b += math.sqrt( lamda[i]*lamda[i] )
    scf.do_diis()
  if mode==7: # u_rsc, eddis
    moldata = lotus_core_hpp.MolData()
    moldata.set_basis_from_file(filename)
    scf.guess_h_core(checkM_a, lamda, moldata.scgtos, moldata.ecps)
    lotus_core_hpp.mat_copy(checkM_b, checkM_a)
    scf.do_ediis()
    scf.u_scf_debug(checkM_a, checkM_b, lamda, lamda_b, moldata, eri, functor)
    abs_error=1.0e-5
    check_a = scf.get_ene_total()
    check_b = 0.0
    for i in range( len(lamda) ):  check_b += math.sqrt( lamda[i]*lamda[i] )
    scf.do_diis()

 
  ret=0
  if math.fabs( check_a - exact_v[0])>abs_error or math.fabs( check_b - exact_v[1] )>abs_error:
    ret=1
    if process_num==0:
      print " ----- in td_dft_mat_pbc_base -----"
      print "   exact_v[0]",exact_v[0],"check_a",check_a,"diff",exact_v[0]-check_a
      print "   exact_v[1]",exact_v[1],"check_b",check_b,"diff",exact_v[1]-check_b

  return ret



class td_scf(unittest.TestCase):
  
  def setUp(self):
    pass

  def test_h_core(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_h_core ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -35.78070460897100,
                  0.00000000000000]
    check = td_scf_base(filename, exact_v, 0, 0, 0, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 0, 0, 0, 1)
    self.assertEqual(check, 0)


  def test_h_core_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_h_core_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -83.2612514756841,
                  0.00000000000000]
    check = td_scf_base(filename_d, exact_v, 0, 0, 0, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename_d, exact_v, 0, 0, 0, 1)
    self.assertEqual(check, 0)


  def test_guess_h_core(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_guess_h_core ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [  29.374285177441323,
                  4.00000000000000]
    check = td_scf_base(filename, exact_v, 1, 0, 0, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 1, 0, 0, 1)
    self.assertEqual(check, 0)

  def test_guess_h_core_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_guess_h_core_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [  43.520839858474474,
                  4.00000000000000]
    check = td_scf_base(filename_d, exact_v, 1, 0, 0, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename_d, exact_v, 1, 0, 0, 1)
    self.assertEqual(check, 0)

  def fun_fock_sub(self, filename, exact_v, sel_functor):
    check = td_scf_base(filename, exact_v, 2, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 2, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 2, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 2, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 2, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 2, 2, sel_functor, 1)
    self.assertEqual(check, 0)

    check = td_scf_base(filename, exact_v, 4, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 4, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 4, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 4, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 4, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 4, 2, sel_functor, 1)
    self.assertEqual(check, 0)

  
  def test_fock_sub_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_hf ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 56.2569474715606,
                56.2569474715606]
    self.fun_fock_sub(filename, exact_v, 0)

  def test_fock_sub_hf_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_hf_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 358.004821713946512,
                358.004821713946512]
    self.fun_fock_sub(filename_d, exact_v, 0)

  def test_fock_sub_slater(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_slater ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 63.7215726251014,
                63.7215726251014]
    self.fun_fock_sub(filename, exact_v, 1)

  def test_fock_sub_slater_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_slater_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 421.3833226925,
                421.3833226925]
    self.fun_fock_sub(filename_d, exact_v, 1)

  def test_fock_sub_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 63.0684437711931,
                63.0684437711931]
    self.fun_fock_sub(filename, exact_v, 3)

  def test_fock_sub_svwn_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_svwn_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 419.581641410476,
                419.581641410476]
    self.fun_fock_sub(filename_d, exact_v, 3)

  def test_fock_sub_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_b3lyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 61.6831388827002,
                61.6831388827002]
    self.fun_fock_sub(filename, exact_v, 5)

  def test_fock_sub_b3lyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_fock_sub_b3lyp_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 407.335528628417,
                407.335528628417]
    self.fun_fock_sub(filename_d, exact_v, 5)

  def fun_scf_sub(self, filename, exact_v, sel_functor):
    check = td_scf_base(filename, exact_v, 3, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 3, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 3, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 3, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 3, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 3, 2, sel_functor, 1)
    self.assertEqual(check, 0)
    
    check = td_scf_base(filename, exact_v, 5, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 5, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 5, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 5, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 5, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 5, 2, sel_functor, 1)
    self.assertEqual(check, 0)

  def test_scf_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_hf ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -15.997926929989090,
                  4.074002074868483]
    self.fun_scf_sub(filename, exact_v, 0)

  def test_scf_hf_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_hf_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -16.049785900583380,
                 23.509691116412167]
    self.fun_scf_sub(filename_d, exact_v, 0)

  def test_scf_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -16.31968247914776,
                  3.021254946143811]
    self.fun_scf_sub(filename, exact_v, 3)
  
  def test_scf_svwn_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_svwn_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -16.38449153649990,
                 19.61019670025221]
    self.fun_scf_sub(filename_d, exact_v, 3)

  def test_scf_b88(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_b88 ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -1.604226179160642e+01,
                 3.061640994030337e+00]
    self.fun_scf_sub(filename, exact_v, 2)

  def test_scf_b88_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_b88_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -1.609867110753819e+01,
                 1.999830828181901e+01]
    self.fun_scf_sub(filename_d, exact_v, 2)

  def test_scf_blyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_blyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -1.628869168612131e+01,
                 2.994283808452439e+00]
    self.fun_scf_sub(filename, exact_v, 4)
 
  def test_scf_blyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_blyp_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -1.634477586490963e+01,
                 1.972139477046332e+01]
    self.fun_scf_sub(filename_d, exact_v, 4)
 
  def test_scf_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_b3lyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -1.632746012006390e+01,
                 3.122207493754346e+00]
    self.fun_scf_sub(filename, exact_v, 5)

  def test_scf_b3lyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_b3lyp_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -1.638284192429592e+01,
                 2.028712129082399e+01]
    self.fun_scf_sub(filename_d, exact_v, 5)

  def fun_scf_sub_ediis(self, filename, exact_v, sel_functor):
    check = td_scf_base(filename, exact_v, 6, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 6, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 6, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 6, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 6, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 6, 2, sel_functor, 1)
    self.assertEqual(check, 0)
    
    check = td_scf_base(filename, exact_v, 7, 0, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 7, 1, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 7, 2, sel_functor, 0)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 7, 0, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 7, 1, sel_functor, 1)
    self.assertEqual(check, 0)
    check = td_scf_base(filename, exact_v, 7, 2, sel_functor, 1)
    self.assertEqual(check, 0)

  def test_scf_hf_ediis(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_hf_ediis ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -15.997926929989090,
                  4.074002074868483]
    self.fun_scf_sub_ediis(filename, exact_v, 0)

  def test_scf_hf_ediis_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_scf_hf_ediis_d ====="
    filename_d = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -16.049785900583380,
                 23.509691116412167]
    self.fun_scf_sub_ediis(filename_d, exact_v, 0)


def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test6.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_scf)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  unittest.TextTestRunner().run(suite)


if __name__ == "__main__":
  run_test()






import unittest
import lotus_core_hpp
import lotus_core
import math
import os


def td_lotus_base2(filename, exact_v, sel_functor, mode_mat):
   
  if mode_mat==0:
    lts     = lotus_core.create_lotus("dMatrix")
  if mode_mat==1:
    lts     = lotus_core.create_lotus("dMatrix_map")

  process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  if process_num==0: print type(lts)

  if sel_functor==0: functor = lotus_core_hpp.HF_Functor()
  if sel_functor==1: functor = lotus_core_hpp.Slater_Functor()
  if sel_functor==2: functor = lotus_core_hpp.B88_Functor()
  if sel_functor==3: functor = lotus_core_hpp.SVWN_Functor()
  if sel_functor==4: functor = lotus_core_hpp.BLYP_Functor()
  if sel_functor==5: functor = lotus_core_hpp.B3LYP_Functor()

  lts.read_in_file(filename)
  lts.env_mol.functor=functor
  lts.guess_h_core()
  lts.scf_py()
  force = lts.cal_force_py()  
 
  check_v = lotus_core_hpp.get_vector_double(4)
  check_v[0] = lts.ene_total
  check_v[1]=0.0
  check_v[2]=0.0
  check_v[3]=0.0
  for a in range( len(force)/3 ):
    check_v[1] += math.sqrt(force[a*3+0]*force[a*3+0]) 
    check_v[2] += math.sqrt(force[a*3+1]*force[a*3+1]) 
    check_v[3] += math.sqrt(force[a*3+2]*force[a*3+2]) 

  ret=0
  abs_error=1.0e-6
  if abs( check_v[0]-exact_v[0])>abs_error or abs(check_v[1]-exact_v[1])>abs_error or \
     abs( check_v[2]-exact_v[2])>abs_error or abs(check_v[3]-exact_v[3])>abs_error:
    ret=1
    if process_num==0:
      print " ----- in td_lotus_base2 -----"
      print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
      print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
      print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]
      print "   exact_v[3]",exact_v[3],"check_v[3]",check_v[3],"diff",exact_v[3]-check_v[3]

  return ret


class td_lotus2(unittest.TestCase):
  
  def setUp(self):
    pass

  def test_hcch_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_hcch_hf ====="
    filename_hcch = os.path.abspath(os.path.dirname(__file__))+"/hcch.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -7.680140602139556e+01;
    exact_v[1] =  0.0;
    exact_v[2] =  0.0;
    exact_v[3] =  2.849170884926731e-01;
    check = td_lotus_base2(filename_hcch, exact_v, 0, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_hcch, exact_v, 0, 1)
    self.assertEqual(check, 0)
  
  def test_hcch_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_hcch_svwn ====="
    filename_hcch = os.path.abspath(os.path.dirname(__file__))+"/hcch.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -7.683983310700512e+01;
    exact_v[1] =  0.0;
    exact_v[2] =  0.0;
    exact_v[3] =  2.756369560779042e-01; 
    check = td_lotus_base2(filename_hcch, exact_v, 3, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_hcch, exact_v, 3, 1)
    self.assertEqual(check, 0)

  def test_hcch_b88(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_hcch_b88 ====="
    filename_hcch = os.path.abspath(os.path.dirname(__file__))+"/hcch.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -7.683293574418317e+01;
    exact_v[1] =  0.0;
    exact_v[2] =  0.0;
    exact_v[3] =  2.719936081708922e-01;
    check = td_lotus_base2(filename_hcch, exact_v, 2, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_hcch, exact_v, 2, 1)
    self.assertEqual(check, 0)
  
  def test_hcch_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_hcch_b3lyp ====="
    filename_hcch = os.path.abspath(os.path.dirname(__file__))+"/hcch.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -7.731144645033552e+01;
    exact_v[1] =  0.0;
    exact_v[2] =  0.0;
    exact_v[3] =  2.730583258161785e-01;
    check = td_lotus_base2(filename_hcch, exact_v, 5, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_hcch, exact_v, 5, 1)
    self.assertEqual(check, 0)

  def test_alanine_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_alanine_hf ====="
    filename_alanine = os.path.abspath(os.path.dirname(__file__))+"/alanine.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -2.481285984093691e+02;
    exact_v[1] =  8.330032027968924e-02;
    exact_v[2] =  1.256274027153565e-01;
    exact_v[3] =  6.616315844422377e-02;
    check = td_lotus_base2(filename_alanine, exact_v, 0, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_alanine, exact_v, 0, 1)
    self.assertEqual(check, 0)

  def test_alanine_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_alanine_b3lyp ====="
    filename_alanine = os.path.abspath(os.path.dirname(__file__))+"/alanine.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -2.496889632812021e+02;
    exact_v[1] =  4.761406741095729e-02;
    exact_v[2] =  5.353982718038575e-02;
    exact_v[3] =  4.609678623533991e-02;
    check = td_lotus_base2(filename_alanine, exact_v, 5, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_alanine, exact_v, 5, 1)
    self.assertEqual(check, 0)

  def test_benzene_hf(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_benzene_hf ====="
    filename_benzene = os.path.abspath(os.path.dirname(__file__))+"/benzene.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -2.306870098715535e+02;
    exact_v[1] =  4.608961796313462e-01;
    exact_v[2] =  3.991697000901296e-01;
    exact_v[3] =  4.798649697481912e-14;
    check = td_lotus_base2(filename_benzene, exact_v, 0, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_benzene, exact_v, 0, 1)
    self.assertEqual(check, 0)

  def test_benzene_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_benzene_b3lyp ====="
    filename_benzene = os.path.abspath(os.path.dirname(__file__))+"/benzene.in"
    exact_v = lotus_core_hpp.get_vector_double(4)
    exact_v[0] = -2.322297719351925e+02;
    exact_v[1] =  4.628016024795332e-01;
    exact_v[2] =  4.008556467263605e-01;
    exact_v[3] =  0.0;
    check = td_lotus_base2(filename_benzene, exact_v, 5, 0)
    self.assertEqual(check, 0)
    check = td_lotus_base2(filename_benzene, exact_v, 5, 1)
    self.assertEqual(check, 0)



def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test9.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_lotus2)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  unittest.TextTestRunner().run(suite)



if __name__ == "__main__":
  run_test()







import unittest
import lotus_core_hpp
import math
import os


def td_dft_mat_base(filename, exact_v, mode_u, mode, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  if mode_mat==0:
    fock1e = lotus_core_hpp.Fock1e()
    ginteg = lotus_core_hpp.Grid_Inte()
    S      = lotus_core_hpp.dMatrix()
    Vxc_a  = lotus_core_hpp.dMatrix()
    Vxc_b  = lotus_core_hpp.dMatrix()
  elif mode_mat==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    ginteg = lotus_core_hpp.Grid_Inte_map()
    S      = lotus_core_hpp.dMatrix_map()
    Vxc_a  = lotus_core_hpp.dMatrix_map()
    Vxc_b  = lotus_core_hpp.dMatrix_map()

  fock1e.cal_S(S, scgtos)
  if mode==0:  functor = lotus_core_hpp.Slater_Functor()
  if mode==1:  functor = lotus_core_hpp.B88_Functor()
  if mode==2:  functor = lotus_core_hpp.SVWN_Functor()
  if mode==3:  functor = lotus_core_hpp.BLYP_Functor()
  if mode==4:  functor = lotus_core_hpp.B3LYP_Functor()

  if mode_u==0:  ene_dft = ginteg.grid_integral_mat(Vxc_a, S, scgtos, functor)
  else:          ene_dft = ginteg.grid_integral_mat_u(Vxc_a, Vxc_b, S, S, scgtos, functor)

  if mode_u==0:
    svxc_a = lotus_core_hpp.Util.cal_DV(S, Vxc_a)
    svxc_b = svxc_a
  else:
    svxc_a = lotus_core_hpp.Util.cal_DV(S, Vxc_a)
    svxc_b = lotus_core_hpp.Util.cal_DV(S, Vxc_b)
   
  ret=0
  if math.fabs( ene_dft - exact_v[0] )>1.0e-8 or \
     math.fabs( svxc_a  - exact_v[1] )>1.0e-8 or \
     math.fabs( svxc_b  - exact_v[2] )>1.0e-8:
    ret=1
    print " ----- error in td_dft_mat_base ----"
    print "   exact_v[0]",exact_v[0]," ene_dft  ",ene_dft," diff ",exact_v[0]-ene_dft
    print "   exact_v[1]",exact_v[1]," svsc_a   ",svxc_a, " diff ",exact_v[1]-svxc_a
    print "   exact_v[2]",exact_v[2]," svsc_a   ",svxc_b, " diff ",exact_v[2]-svxc_b

  return ret

  

def td_dft_grad_base(filename, exact_v, mode_u, mode, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  if mode_mat==0:
    fock1e = lotus_core_hpp.Fock1e()
    ginteg = lotus_core_hpp.Grid_Inte()
    S      = lotus_core_hpp.dMatrix()
  elif mode_mat==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    ginteg = lotus_core_hpp.Grid_Inte_map()
    S      = lotus_core_hpp.dMatrix_map()

  fock1e.cal_S(S, scgtos)
  if mode==0:  functor = lotus_core_hpp.Slater_Functor()
  if mode==1:  functor = lotus_core_hpp.B88_Functor()
  if mode==2:  functor = lotus_core_hpp.SVWN_Functor()
  if mode==3:  functor = lotus_core_hpp.BLYP_Functor()
  if mode==4:  functor = lotus_core_hpp.B3LYP_Functor()

  if mode_u==0:  grad = ginteg.cal_grad(scgtos, S, functor)
  else:          grad = ginteg.cal_grad_u(scgtos, S, S, functor)

  check_v = lotus_core_hpp.get_vector_double(3)
  for i in range(3): check_v[i]=0.0

  for a in range( len(grad)/3 ):
    check_v[0] += math.sqrt( grad[a*3+0]*grad[a*3+0] )
    check_v[1] += math.sqrt( grad[a*3+1]*grad[a*3+1] )
    check_v[2] += math.sqrt( grad[a*3+2]*grad[a*3+2] )

  ret=0
  if math.fabs( check_v[0] - exact_v[0])>1.0e-12 or \
     math.fabs( check_v[1] - exact_v[1])>1.0e-12 or \
     math.fabs( check_v[2] - exact_v[2])>1.0e-12:
    ret=1
    print " ----- in td_dft_grad_base -----"
    print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
    print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
    print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]

  return ret

   

class td_dft(unittest.TestCase):
  
  def setUp(self):
    pass

  def fun_check(self, filename, exact_v, sel):
    check = td_dft_mat_base(filename, exact_v, 0, sel, 0)
    self.assertEqual(check, 0)
    check = td_dft_mat_base(filename, exact_v, 0, sel, 1)
    self.assertEqual(check, 0)
    check = td_dft_mat_base(filename, exact_v, 1, sel, 0)
    self.assertEqual(check, 0)
    check = td_dft_mat_base(filename, exact_v, 1, sel, 1)
    self.assertEqual(check, 0)


  # slater
  def test_dft_mat_slater(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_slater ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [-8.68918462925892,
               -5.79278975283834,
               -5.79278975283834]
    self.fun_check(filename, exact_v, 0)

  def test_dft_mat_slater_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_slater_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [-28.7225990726249,
               -19.1483993817468,
               -19.1483993817468]
    self.fun_check(filename, exact_v, 0)

  # b88
  def test_dft_mat_b88(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_b88 ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [-9.2876235408606,
               -6.03693727770248,
               -6.03693727770248]
    self.fun_check(filename, exact_v, 1)

  def test_dft_mat_b88_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_b88_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [-29.8278342726607,
               -19.5962495878738,
               -19.5962495878738]
    self.fun_check(filename, exact_v, 1)

  # svwn
  def test_dft_mat_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [-9.87546262859488,
               -6.4459186067467,
               -6.4459186067467]
    self.fun_check(filename, exact_v, 2)

  def test_dft_mat_svwn_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_svwn_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [-32.0105268520034,
               -20.9500806637708,
               -20.9500806637708]
    self.fun_check(filename, exact_v, 2)

  # blyp
  def test_dft_mat_blyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_blyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [-9.77158715290214,
               -6.34018173640426,
               -6.34018173640426]
    self.fun_check(filename, exact_v, 3)

  def test_dft_mat_blyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_blyp_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [-31.3144744951157,
               -20.4694869661171,
               -20.4694869661171]
    self.fun_check(filename, exact_v, 3)

  # b3lyp
  def test_dft_mat_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_b3lyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [-7.99962706538783,
               -5.17974051396373,
               -5.17974051396373]
    self.fun_check(filename, exact_v, 4)

  def test_dft_mat_b3lyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_mat_b3lyp_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [-25.6027334603965,
               -16.69081337377,
               -16.69081337377]
    self.fun_check(filename, exact_v, 4)

  #
  # gradient
  #
             
  def fun_grad_check(self, filename, exact_v, sel):
    check = td_dft_grad_base(filename, exact_v, 0, sel, 0)
    self.assertEqual(check, 0)
    check = td_dft_grad_base(filename, exact_v, 0, sel, 1)
    self.assertEqual(check, 0)
    check = td_dft_grad_base(filename, exact_v, 1, sel, 0)
    self.assertEqual(check, 0)
    check = td_dft_grad_base(filename, exact_v, 1, sel, 1)
    self.assertEqual(check, 0)

 
  # slater
  def test_dft_grad_slater(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_slater ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 1.05296460949943,
                0.00000000000000,
                1.55725111110006]
    self.fun_grad_check(filename, exact_v, 0)

  def test_dft_grad_slater_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_slater_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.87484447384673,
                0.00000000000000,
                4.16771240491097]
    self.fun_grad_check(filename, exact_v, 0)

  # b88
  def test_dft_grad_b88(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_b88 ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 1.04847620377321,
                0.00000000000000,
                1.56017486613892]
    self.fun_grad_check(filename, exact_v, 1)

  def test_dft_grad_b88_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_b88_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.91520808450086,
                0.00000000000000,
                4.22884556905654]
    self.fun_grad_check(filename, exact_v, 1)

  # svwn
  def test_dft_grad_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 1.14533141423903,
                0.00000000000000,
                1.69443739285219]
    self.fun_grad_check(filename, exact_v, 2)

  def test_dft_grad_svwn_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_svwn_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 3.08784188862778,
                0.00000000000000,
                4.4766019037608]
    self.fun_grad_check(filename, exact_v, 2)

  # blyp
  def test_dft_grad_blyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_blyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 1.10246933713149,
                0.00000000000000,
                1.63759463570879]
    self.fun_grad_check(filename, exact_v, 3)

  def test_dft_grad_blyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_blyp_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 3.02653535330202,
                0.00000000000000,
                4.38957440364052]
    self.fun_grad_check(filename, exact_v, 3)

  # b3lyp
  def test_dft_grad_b3lyp(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_b3lyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ 0.900424166397432,
                0.000000000000000,
                 1.3366813993925]
    self.fun_grad_check(filename, exact_v, 4)

  def test_dft_grad_b3lyp_d(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_grad_b3lyp_d ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ 2.45958197528588,
                0.00000000000000,
                3.56706516290775]
    self.fun_grad_check(filename, exact_v, 4)



def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test3.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_dft)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  testRunner.run(suite)


if __name__ == "__main__":
  run_test()





import unittest
import test1
import test2
import test3
import test4
import test5
import test6
import test7
import test8
import test9


def suite(): 
  suite = unittest.TestSuite()
  suite.addTests(unittest.makeSuite(test1.td_fock1e))
  suite.addTests(unittest.makeSuite(test2.td_fock2e))
  suite.addTests(unittest.makeSuite(test3.td_dft))
  suite.addTests(unittest.makeSuite(test4.td_dft_pbc))
  suite.addTests(unittest.makeSuite(test5.td_matrix))
  suite.addTests(unittest.makeSuite(test6.td_scf))
  suite.addTests(unittest.makeSuite(test7.td_grad))
  suite.addTests(unittest.makeSuite(test8.td_lotus))
  suite.addTests(unittest.makeSuite(test9.td_lotus2))
  return suite







import unittest
import lotus_core_hpp
import lotus_core
import math
import os


def td_lotus_base(filename, exact_v, mode, sel_functor, mode_mat):

  if mode_mat==0:
    lts     = lotus_core.create_lotus("dMatrix")
    S       = lotus_core_hpp.dMatrix()
    M       = lotus_core_hpp.dMatrix()
  if mode_mat==1:
    lts     = lotus_core.create_lotus("dMatrix_map")
    S       = lotus_core_hpp.dMatrix_map()
    M       = lotus_core_hpp.dMatrix_map()

  os.chdir( os.path.abspath(os.path.dirname(__file__)) )

  lts.read_in_file_debug(filename)
  lts.cal_matrix("S", S)
  lts.guess_h_core()

  check_v = lotus_core_hpp.get_vector_double(10)
  abs_error=1.0e-8
  if mode==0:
    check_v[0] = lts.get_N_cgtos() 
    check_v[1] = lts.get_N_ecps() 
    check_v[2] = lts.get_N_atoms() 
    lts.cal_matrix("K", M)
    check_v[3] = lotus_core_hpp.Util.cal_DV(S, M)
    lts.cal_matrix("NA", M)
    check_v[4] = lotus_core_hpp.Util.cal_DV(S, M)
    lts.cal_matrix("ECP", M)
    check_v[5] = lotus_core_hpp.Util.cal_DV(S, M)
    lts.cal_matrix("H_core", M)
    check_v[6] = lotus_core_hpp.Util.cal_DV(S, M)
    lts.cal_matrix("D", M)
    check_v[7] = lotus_core_hpp.Util.cal_DV(S, M)
    lts.cal_matrix("D_a", M)
    check_v[8] = lotus_core_hpp.Util.cal_DV(S, M)

  ret=0
  abs_error=5.0e-11
  if abs( check_v[0]-exact_v[0])>abs_error or abs(check_v[1]-exact_v[1])>abs_error or \
     abs( check_v[2]-exact_v[2])>abs_error or abs(check_v[3]-exact_v[3])>abs_error or \
     abs( check_v[4]-exact_v[4])>abs_error or abs(check_v[5]-exact_v[5])>abs_error or \
     abs( check_v[6]-exact_v[6])>abs_error or abs(check_v[7]-exact_v[7])>abs_error or \
     abs( check_v[8]-exact_v[8])>abs_error:
    ret=1
    print " ----- in td_lotus_base -----"
    print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
    print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
    print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]
    print "   exact_v[3]",exact_v[3],"check_v[3]",check_v[3],"diff",exact_v[3]-check_v[3]
    print "   exact_v[4]",exact_v[4],"check_v[4]",check_v[4],"diff",exact_v[4]-check_v[4]
    print "   exact_v[5]",exact_v[5],"check_v[5]",check_v[5],"diff",exact_v[5]-check_v[5]
    print "   exact_v[6]",exact_v[6],"check_v[6]",check_v[6],"diff",exact_v[6]-check_v[6]
    print "   exact_v[7]",exact_v[7],"check_v[7]",check_v[7],"diff",exact_v[7]-check_v[7]
    print "   exact_v[8]",exact_v[8],"check_v[8]",check_v[8],"diff",exact_v[8]-check_v[8]

  return ret


class td_lotus(unittest.TestCase):
  
  def setUp(self):
    pass

  def test_lotus(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_lotus_matrix ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d_for_debug.in"
    exact_v = lotus_core_hpp.get_vector_double(10)
    exact_v[0] =  12.0
    exact_v[1] =   2.0
    exact_v[2] =   3.0
    exact_v[3] =  35.305943879326982
    exact_v[4] =-120.209150645409
    exact_v[5] =   1.64195529039806
    exact_v[6] = -83.2612514756841
    exact_v[7] =   4.0
    exact_v[8] =   4.0
    check = td_lotus_base(filename, exact_v, 0, 0, 0) 
    self.assertEqual(check, 0)


def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test8.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_lotus)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  unittest.TextTestRunner().run(suite)


if __name__ == "__main__":
  run_test()





import unittest
import lotus_core_hpp
import math
import os


def td_dft_pbc_mat_base(filename, exact_v, mode, mode_mat):
  scgtos = lotus_core_hpp.Util_GTO.get_scgtos_from_file(filename)
  ecps   = lotus_core_hpp.ECP.get_ecps_from_file(filename)

  ctrl_pbc = lotus_core_hpp.CTRL_PBC()
  T123 = lotus_core_hpp.get_vector_double(9)
  T123[0] =  1.0 
  T123[1] = -0.5 
  T123[2] =  0.0 
  T123[3] =  0.0 
  T123[4] = -1.5 
  T123[5] =  0.0 
  T123[6] =  0.0 
  T123[7] =  1.5 
  T123[8] =  1.0 
  max_Nc = lotus_core_hpp.get_vector_int(3)
  ctrl_pbc.set_T123(T123)
#  ctrl_pbc.show()
  
  if mode_mat==0:
    fock1e = lotus_core_hpp.Fock1e()
    ginteg = lotus_core_hpp.Grid_Inte()
    S_PBC  = lotus_core_hpp.get_vector_dMatrix()
    M_PBC  = lotus_core_hpp.get_vector_dMatrix()
  elif mode_mat==1:
    fock1e = lotus_core_hpp.Fock1e_map()
    ginteg = lotus_core_hpp.Grid_Inte_map()
    S_PBC  = lotus_core_hpp.get_vector_dMatrix_map()
    M_PBC  = lotus_core_hpp.get_vector_dMatrix_map()

#  print "*********** DEBUG mode_mat=",mode_mat," type ",type(ginteg), type(M_PBC)

  if mode==0:  functor = lotus_core_hpp.Slater_Functor()
  if mode==1:  functor = lotus_core_hpp.B88_Functor()
  if mode==2:  functor = lotus_core_hpp.SVWN_Functor()
  if mode==3:  functor = lotus_core_hpp.BLYP_Functor()
  if mode==4:  functor = lotus_core_hpp.B3LYP_Functor()

  check_v = lotus_core_hpp.get_vector_double(10)

  max_Nc[0] = 0
  max_Nc[1] = 0
  max_Nc[2] = 0
  ctrl_pbc.set_max_Nc(max_Nc)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)
  check_v[0] = ginteg.grid_integral_mat_PBC(M_PBC, S_PBC, scgtos, ctrl_pbc, functor)
  check_v[1] = lotus_core_hpp.Util_PBC.cal_DV(M_PBC, S_PBC, ctrl_pbc.get_N123_c() ) 

  max_Nc[0] = 1
  max_Nc[1] = 0
  max_Nc[2] = 0
  ctrl_pbc.set_max_Nc(max_Nc)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)
  check_v[2] = ginteg.grid_integral_mat_PBC(M_PBC, S_PBC, scgtos, ctrl_pbc, functor)
  check_v[3] = lotus_core_hpp.Util_PBC.cal_DV(M_PBC, S_PBC, ctrl_pbc.get_N123_c() ) 

  max_Nc[0] = 1
  max_Nc[1] = 1
  max_Nc[2] = 0
  ctrl_pbc.set_max_Nc(max_Nc)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)
  check_v[4] = ginteg.grid_integral_mat_PBC(M_PBC, S_PBC, scgtos, ctrl_pbc, functor)
  check_v[5] = lotus_core_hpp.Util_PBC.cal_DV(M_PBC, S_PBC, ctrl_pbc.get_N123_c() ) 

  max_Nc[0] = 1
  max_Nc[1] = 1
  max_Nc[2] = 1
  ctrl_pbc.set_max_Nc(max_Nc)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)
  check_v[6] = ginteg.grid_integral_mat_PBC(M_PBC, S_PBC, scgtos, ctrl_pbc, functor)
  check_v[7] = lotus_core_hpp.Util_PBC.cal_DV(M_PBC, S_PBC, ctrl_pbc.get_N123_c() ) 

  max_Nc[0] = 0
  max_Nc[1] = 2
  max_Nc[2] = 2
  ctrl_pbc.set_max_Nc(max_Nc)
  fock1e.cal_S_PBC(S_PBC, scgtos, ctrl_pbc)
  check_v[8] = ginteg.grid_integral_mat_PBC(M_PBC, S_PBC, scgtos, ctrl_pbc, functor)
  check_v[9] = lotus_core_hpp.Util_PBC.cal_DV(M_PBC, S_PBC, ctrl_pbc.get_N123_c() ) 

  ret=0
  abs_error=1.0e-9
  if math.fabs( check_v[0] - exact_v[0])>abs_error or math.fabs( check_v[1] - exact_v[1] )>abs_error or \
     math.fabs( check_v[2] - exact_v[2])>abs_error or math.fabs( check_v[3] - exact_v[3] )>abs_error or \
     math.fabs( check_v[4] - exact_v[4])>abs_error or math.fabs( check_v[5] - exact_v[5] )>abs_error or \
     math.fabs( check_v[6] - exact_v[6])>abs_error or math.fabs( check_v[7] - exact_v[7] )>abs_error or \
     math.fabs( check_v[8] - exact_v[8])>abs_error or math.fabs( check_v[9] - exact_v[9] )>abs_error:
    ret=1
    print " ----- in td_dft_mat_pbc_base -----"
    print "   exact_v[0]",exact_v[0],"check_v[0]",check_v[0],"diff",exact_v[0]-check_v[0]
    print "   exact_v[1]",exact_v[1],"check_v[1]",check_v[1],"diff",exact_v[1]-check_v[1]
    print "   exact_v[2]",exact_v[2],"check_v[2]",check_v[2],"diff",exact_v[2]-check_v[2]
    print "   exact_v[3]",exact_v[3],"check_v[3]",check_v[3],"diff",exact_v[3]-check_v[3]
    print "   exact_v[4]",exact_v[4],"check_v[4]",check_v[4],"diff",exact_v[4]-check_v[4]
    print "   exact_v[5]",exact_v[5],"check_v[5]",check_v[5],"diff",exact_v[5]-check_v[5]
    print "   exact_v[6]",exact_v[6],"check_v[6]",check_v[6],"diff",exact_v[6]-check_v[6]
    print "   exact_v[7]",exact_v[7],"check_v[7]",check_v[7],"diff",exact_v[7]-check_v[7]
    print "   exact_v[8]",exact_v[8],"check_v[8]",check_v[8],"diff",exact_v[8]-check_v[8]
    print "   exact_v[9]",exact_v[9],"check_v[9]",check_v[9],"diff",exact_v[9]-check_v[9]

  return ret



class td_dft_pbc(unittest.TestCase):
  
  def setUp(self):
    pass
  
  # slater
  def test_dft_pbc_mat_slater(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_pbc_mat_slater ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc.mol"
    exact_v = [ -8.68918462925892,
                -5.79278975283924,
               -27.9470595016986,
               -18.3924381013131,
               -68.2891368400923,
               -40.7319674149235,
              -247.966315093019,
              -138.684446232039,
               -66.701110932087,
               -44.9884398391664]
    check = td_dft_pbc_mat_base(filename, exact_v, 0, 0)
    self.assertEqual(check, 0)
    check = td_dft_pbc_mat_base(filename, exact_v, 0, 1)
    self.assertEqual(check, 0)

  # svwn
  def test_dft_pbc_mat_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_pbc_mat_svwn ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -32.0105268520034,
                -20.9500806637737,
               -118.27853809234,
                -73.9886286096848,
               -345.455974988649,
               -195.890365412771,
              -1259.35881653829,
               -674.86594442786,
               -360.996815911785,
               -240.620223087907]
    check = td_dft_pbc_mat_base(filename, exact_v, 2, 0)
    self.assertEqual(check, 0)
    check = td_dft_pbc_mat_base(filename, exact_v, 2, 1)
    self.assertEqual(check, 0)
  
  # blyp
  def test_dft_pbc_mat_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_pbc_mat_blyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -31.3144744951157,
                -20.4694869661198,
               -115.190027883041,
                -72.1824246518356,
               -336.759674739418,
               -191.604790643238,
              -1234.31848438909,
               -663.667927459116,
               -352.107236868374,
               -235.516086217432]
    check = td_dft_pbc_mat_base(filename, exact_v, 3, 0)
    self.assertEqual(check, 0)
    check = td_dft_pbc_mat_base(filename, exact_v, 3, 1)
    self.assertEqual(check, 0)
  
  # b3lyp
  def test_dft_pbc_mat_svwn(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dft_pbc_mat_b3lyp ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_sbkjc_plus_d.mol"
    exact_v = [ -25.6027334603965,
                -16.6908133737727,
                -93.6980535024515,
                -58.5599510952698,
               -272.91806864864,
               -154.938239837276,
               -996.492587711361,
               -534.829799499708,
               -285.197210945696,
               -190.333328397354]
    check = td_dft_pbc_mat_base(filename, exact_v, 4, 0)
    self.assertEqual(check, 0)
    check = td_dft_pbc_mat_base(filename, exact_v, 4, 1)
    self.assertEqual(check, 0)


def run_test():
  try:
    from mpi4py import MPI
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
  except:
    lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"

  if process_num==0: print " ##### test4.py #####"
  suite = unittest.TestLoader().loadTestsFromTestCase(td_dft_pbc)
  try:
    import xmlrunner
    testRunner = xmlrunner.XMLTestRunner()
  except:
    if process_num==0: print " unittest-xml-reporting is not installed"
    testRunner = unittest.TextTestRunner()

  testRunner.run(suite)


if __name__ == "__main__":
  run_test()






import test1
import test2
import test3
import test4
import test5
import test6
import test7
import test8
import test9


def do_test(no):
  if no==1: test1.run_test()
  if no==2: test2.run_test()
  if no==3: test3.run_test()
  if no==4: test4.run_test()
  if no==5: test5.run_test()
  if no==6: test6.run_test()
  if no==7: test7.run_test()
  if no==8: test8.run_test()
  if no==9: test9.run_test()

def self_tests(li):
  for no in li:
    do_test(no)

def self_tests_all():
  li = [1, 2, 3, 4, 5, 6, 7, 8, 9]
  self_tests(li) 






from setuptools import setup, Extension, find_packages
import sys
import os

from distutils import sysconfig
import subprocess 

sys.path.append("./lotus_core")
sys.path.append("./tests_py")




def compile_python4lotus():
  pyver = sysconfig.get_config_var('VERSION')
  getvar = sysconfig.get_config_var
  flags = ['-I' + sysconfig.get_python_inc(),
           '-I' + sysconfig.get_python_inc(plat_specific=True)]
  libs = getvar('LIBS').split() + getvar('SYSLIBS').split()
  libs.append('-lpython'+pyver)

  str_flags = " ".join(flags)
  str_libs  = " ".join(libs)

  cmd_CC  ="gcc"
  cmd_CXX ="g++"
  if "CC"  in os.environ:  cmd_CC  = os.environ["CC"]
  if "CXX" in os.environ:  cmd_CXX = os.environ["CXX"]
  cmd=cmd_CXX+" "+str_flags+" -o bin/python4lotus src/main_python4lotus.cpp "+str_libs
  print "===== compile python4lotus ====="
  print "\n",cmd,"\n"
  if os.path.exists("bin/python4lotus")==True: os.remove("bin/python4lotus")
  if os.path.exists("bin")==False: os.mkdir("bin")
  subprocess.call(cmd.split())


compile_python4lotus()

module = Extension(
  "lotus_core_hpp",
  sources            = ["src/module_core.cpp"],
  include_dirs       = ["include_hpp","include_py","pint/include_hpp"],
  extra_compile_args = ["-w"],
  libraries          = ["boost_python"]
)

setup(
  name               = "lotus_core",
  packages           = ["lotus_core"],
  version            = "0.0.6",
  package_data       = {"lotus_core":["basis_DATA/*.basis","include_hpp/*.hpp",\
                                      "include_hpp/detail/*.hpp",\
                                      "include_py/*.hpp",\
                                      "pint/include_hpp/*.hpp",\
                                      "pint/include_hpp/*.h",\
                                      "pint/include_hpp/detail/*.hpp",\
                                      "pint/include_hpp/detail/*.h",\
                                      "pint/include_py/*.hpp",\
                                      "tests_py/*.py",\
                                      "tests_py/*.in",\
                                      "tests_py/*.mol"]},
  ext_modules        = [module],
  scripts            = ["bin/python4lotus"],
  author             = "T.Shimazaki, T.Abe, M.Hashimoto, and T.Maeda",
  author_email       = "t-shimazaki@riken.jp",
  license            = "MIT License",
  url                = "https://bitbucket.org/tshima/lotus_core",
  classifiers        = ["License :: OSI Approved :: MIT License",
                        "Operating System :: MacOS :: MacOS X", 
                        "Operating System :: POSIX :: Linux", 
                       ],

  test_suite         = "test_suite.suite"
)





import unittest
import lotus_core
import lotus_core_hpp
import math
import os


def check_M(A, X, lamda):
  N_cgtos = len(lamda)
  Ax = lotus_core_hpp.get_vector_double(N_cgtos)

  abs_error=0.0
  for q in range(N_cgtos):
    for i in range(N_cgtos):
      tmp_v_i = 0.0
      for j in range(N_cgtos):
        tmp_v_i += A.get_value(i,j)*X.get_value(j,q)
      Ax[i]=tmp_v_i

    for i in range(N_cgtos):
      tmp_c = Ax[i] - lamda[q]*X.get_value(i,q)
      abs_error += abs(tmp_c)
  
  return abs_error



def check_M2(A, S, X, lamda, SX):
  N_cgtos = len(lamda)
  Ax = []
  [Ax.append(0.0) for i in range(N_cgtos)]
  SX.set_IJ(N_cgtos, N_cgtos)
  lotus_core_hpp.mat_mul(SX, S, X)

  abs_error=0.0
  for q in range(N_cgtos):
    for i in range(N_cgtos):
      tmp_v_i = 0.0
      for j in range(N_cgtos):
        tmp_v_i += A.get_value(i,j)*X.get_value(j,q)
      Ax[i]=tmp_v_i

    for i in range(N_cgtos):
      tmp_c = Ax[i] - lamda[q]*SX.get_value(i,q)
      abs_error += abs(tmp_c)
  
  return abs_error


def td_lapack_base(filename, mode, mode_mat):

  if mode_mat==0:
    lts    = lotus_core.create_lotus()
    H_core = lotus_core_hpp.dMatrix()
    S      = lotus_core_hpp.dMatrix()
    inv_S  = lotus_core_hpp.dMatrix()
    M      = lotus_core_hpp.dMatrix()
    M2     = lotus_core_hpp.dMatrix()
    SX     = lotus_core_hpp.dMatrix()
    X      = lotus_core_hpp.dMatrix()
    X2     = lotus_core_hpp.dMatrix()
  if mode_mat==1:
    lts    = lotus_core.create_lotus("dMatrix_map")
    H_core = lotus_core_hpp.dMatrix_map()
    S      = lotus_core_hpp.dMatrix_map()
    inv_S  = lotus_core_hpp.dMatrix_map()
    M      = lotus_core_hpp.dMatrix_map()
    M2     = lotus_core_hpp.dMatrix_map()
    SX     = lotus_core_hpp.dMatrix_map()
    X      = lotus_core_hpp.dMatrix_map()
    X2     = lotus_core_hpp.dMatrix_map()

  lamda  = lotus_core_hpp.get_vector_double()
  lamda2 = lotus_core_hpp.get_vector_double()

  lts.read_in_file(filename)
  lts.cal_matrix("H_core", H_core)
  lts.cal_matrix("S",      S)

  ret=0
  abs_error=1.0e-11

  if mode==0:
    lts.cal_matrix("S", inv_S)
    lotus_core.Wrapper4Lapack().inverse(inv_S)
    I=S.get_I()
    M.set_IJ(I,I)
    lotus_core_hpp.mat_mul(M, S, inv_S)
    for i in range(I):
      for j in range(I):
        if i==j:
          if abs(M.get_value(i,j)-1.0)>abs_error: ret=1
        else:
          if abs(M.get_value(i,j)-0.0)>abs_error: ret=1
    lotus_core_hpp.mat_mul(M, inv_S, S)
    for i in range(I):
      for j in range(I):
        if i==j:
          if abs(M.get_value(i,j)-1.0)>abs_error: ret=1
        else:
          if abs(M.get_value(i,j)-0.0)>abs_error: ret=1

  if mode==1:
    I=S.get_I()
    M.set_IJ(I,I/2)
    for i in range(I):
      for j in range(I/2):
        M.set_value(i,j, S.get_value(i,j))

    lotus_core.Wrapper4Lapack().dgesv(H_core, X, M)

    lts.cal_matrix("H_core", M2)
    lotus_core.Wrapper4Lapack().inverse(M2)
    lotus_core_hpp.mat_mul(M2, M2, M) 
    
    for i in range(I):
      for j in range(I/2):
        if abs(X.get_value(i,j)-M2.get_value(i,j))>abs_error:
          ret=1

  if mode==2:
    lotus_core_hpp.cal_eigen(H_core, X, lamda)
    lotus_core.Wrapper4Lapack().dsyev(H_core, X2, lamda2)
    I=X.get_I()
    check=check_M(H_core, X, lamda)
    if check>abs_error:    ret=1
    check=check_M(H_core, X2, lamda2)
    if check>abs_error:  ret=1
    for i in range(I):
      if abs(lamda[i]-lamda2[i])>abs_error:
        print "   ERROR i=",i,"lamda",lamda[i],lamda2[i],"diff",lamda[i]-lamda2[i]
        ret=1

  if mode==3:
    lotus_core_hpp.cal_eigen(H_core, S, X, lamda)
    lotus_core.Wrapper4Lapack().dsygv(H_core, S, X2, lamda2)
    I=X.get_I()
    check=check_M2(H_core, S, X, lamda, SX)
    if check>abs_error:    ret=1
    check=check_M2(H_core, S, X2, lamda2, SX)
    if check>abs_error:  ret=1
    for i in range(I):
      if abs(lamda[i]-lamda2[i])>abs_error:
        print "   ERROR i=",i,"lamda",lamda[i],lamda2[i],"diff",lamda[i]-lamda2[i]
        ret=1

  return ret 


class td_lapack(unittest.TestCase):
  
  def setUp(self):
    pass

  def test_inverse(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_inverse ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_6-31g.in"
    check = td_lapack_base(filename, 0, 0) 
    self.assertEqual(check, 0)
    check = td_lapack_base(filename, 0, 1) 
    self.assertEqual(check, 0)


  def test_dgesv(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dgesv ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_6-31g.in"
    check = td_lapack_base(filename, 1, 0) 
    self.assertEqual(check, 0)
    check = td_lapack_base(filename, 1, 1) 
    self.assertEqual(check, 0)


  def test_dsyev(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dsyev ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_6-31g.in"
    check = td_lapack_base(filename, 2, 0) 
    self.assertEqual(check, 0)
    check = td_lapack_base(filename, 2, 1) 
    self.assertEqual(check, 0)

  def test_dsygv(self):
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print "\n===== test_dsygv ====="
    filename = os.path.abspath(os.path.dirname(__file__))+"/h2o_6-31g.in"
    check = td_lapack_base(filename, 3, 0) 
    self.assertEqual(check, 0)
    check = td_lapack_base(filename, 3, 1) 
    self.assertEqual(check, 0)




if __name__ == "__main__":
  try:
    from mpi4py import MPI
  except:
    process_num = lotus_core_hpp.Util_MPI.get_mpi_rank()
    if process_num==0: print " mpi4py is not installed"
  lotus_core_hpp.Util_MPI.set_MPI_COMM_LOTUS()

  unittest.main()

